const tuyapi = require('tuyapi');
const debug = require('debug')('homebridge-tuya');

const EventEmitter = require('events');

let PlatformAccessory;
let Accessory;
let Service;
let Characteristic;
let UUIDGen;

class TuyaDevices extends EventEmitter {
  constructor (platform, api, config) {
    super();

    this.platform = platform;
    this.api = api;
    this.config = config;
  }

  async discover () {
    for (const device of this.config) {
      if (!this.platform.homebridgeAccessories.get(this.platform.api.hap.uuid.generate(device.devId))) {
        if (device.ip) {
          let TuyaDevice = new HomebridgeTuya(this.platform, {id: device.devId, key: device.localKey, ip: device.ip});
          this.platform.registerPlatformAccessory(TuyaDevice);
        }
        else {
          let TuyaDevice = new HomebridgeTuya(this.platform, {id: device.devId, key: device.localKey});
          await TuyaDevice.device.resolveId();

          this.platform.registerPlatformAccessory(TuyaDevice.homebridgeAccessory);
        }
      }
    }
  }
}

class HomebridgeTuya {
  constructor (platform, config, homebridgeAccessory) {

    if (typeof config === 'string')
    this.device = new tuyapi(config);

    console.log('new tuya device was created!')

    PlatformAccessory = platform.api.platformAccessory;
    Accessory = platform.api.hap.Accessory;
    Service = platform.api.hap.Service;
    Characteristic = platform.api.hap.Characteristic;
    UUIDGen = platform.api.hap.uuid;

    this.log = platform.log;

    this.homebridgeAccessory = new PlatformAccessory(config.id, UUIDGen.generate(config.id), Accessory.Categories.OUTLET);

    this.Service = this.homebridgeAccessory.getService(Service.Outlet);

    if (!this.Service) {
      this.Service = this.homebridgeAccessory.addService(Service.Outlet, config.id);
    }
    else {
      this.Service.setCharacteristic(Characteristic.Name, config.id);
    }

    this.Service.getCharacteristic(Characteristic.On)
      .on('get', callback => {
        this.log.debug('Getting device status...');
        this.device.get().then(status => {
          callback(null, status);
        }).catch(error => {
          callback(error, null);
        });
      })
      .on('set', (value, callback) => {
        this.device.set({set: value}).then(() => {
          callback(null, true);
        }).catch(error => {
          callback(error, null);
        })
      });

    this.homebridgeAccessory.on('identify', this.identify.bind(this));
  }

  getServices () {
    return [this.Service];
  }

  identify (paired, callback) {
    this.log.debug('identified oadsfhii')
    callback();
  }
}

module.exports = {TuyaDevices: TuyaDevices, HomebridgeTuya: HomebridgeTuya};
